--
-- AggroPlates - Colorize the standard nameplates based on threat.
--
-- Simple Threat Colorize on original Blizzard Nameplates. (with ClassColorize on friendly Nameplates)
-- Copyright 2017 by Dominik Hazoth - Hazoth.net work - www.AHYO.de
--

local isTank = function(t)
    return UnitGroupRolesAssigned(t) == "TANK"
end

local isOfftanked = function(frame)
	local t2 = frame.displayedUnit.."target"

    if UnitPlayerOrPetInRaid(t2) or UnitPlayerOrPetInParty(t2) then
        if not UnitIsUnit("player", t2) and isTank(t2) and isTank("player") then
            return true
        end
    end

    return false
end

local threatColor = function(s)
	if isTank("player") or not IsInGroup() then
		if s == 0 then
			return 1.0, 0.0, 0.0	-- Red Color
		elseif s == 1 then		-- yellow Color (Can be deleted)
			return 1.0, 0.5, 0.0	-- yellow Color (Can be deleted)
		elseif s == 2 then		-- orange Color (Can be deleted)
			return 1.0, 1.0, 0.0	-- orange Color (Can be deleted)
		else				-- First AggroColor Green
			return 0.0, 1.0, 0.0	-- First AggroColor Green
		end
	else
		if s == 0 then
			return 0.0, 1.0, 0.0	-- Green Color for DDs
		elseif s == 1 then		-- yellow Color (Can be deleted)
			return 1.0, 1.0, 0.0	-- yellow Color (Can be deleted)
		elseif s == 2 then		-- orange Color (Can be deleted)
			return 1.0, 0.5, 0.0	-- orange Color (Can be deleted)
		else				-- Red Color
			return 1.0, 0.0, 0.0	-- Red Color
		end
	end
end



-------------------------------------
-- Do not change anything from here!
-------------------------------------

local canSee = function(t)
	return UnitBuff(t, "Detector") ~= nil or UnitBuff(t, "On High Alert") ~= nil or UnitBuff(t, "Demonic Vision") ~= nil
end

hooksecurefunc("CompactUnitFrame_UpdateHealthColor", function(frame)
	local r, g, b

	if frame == nil then
		return
	elseif frame.displayedUnit == nil then
		return
	elseif C_NamePlate.GetNamePlateForUnit(frame.unit) == C_NamePlate.GetNamePlateForUnit("player") then
		return
	elseif not UnitIsConnected(frame.displayedUnit) then
		r, g, b = 0.5, 0.5, 0.5
        else
		local col = RAID_CLASS_COLORS[select(2, UnitClass(frame.displayedUnit))]

		if CompactUnitFrame_IsTapDenied(frame) then
			r, g, b = 0.3, 0.3, 0.3
		elseif UnitIsPlayer(frame.displayedUnit) and col then
			r, g, b = col.r, col.g, col.b
		elseif CompactUnitFrame_IsOnThreatListWithPlayer(frame.displayedUnit) then
			local s = UnitThreatSituation("player", frame.displayedUnit)

			if isOfftanked(frame) then
				if s > 0 then
					r, g, b = 1.0, 0.5, 1.0
				else
					r, g, b = 0.0, 1.0, 0.8
				end
			else
				r, g, b = threatColor(s)
			end
		elseif isOfftanked(frame) then
				r, g, b = 0.0, 1.0, 1.0
		elseif canSee(frame.displayedUnit) then
			r, g, b = 0.0, 1.0, 1.0
		else
			local reaction = UnitReaction("player", frame.displayedUnit)
			if reaction <= 3 then
 				r, g, b = 1.0, 0.7, 0.7
			elseif reaction == 4 then
				local re, ge, be = UnitSelectionColor(frame.displayedUnit)
				if ge == 0 and be == 0 then
					r, g, b = 1.0, 0.7, 0.7
				else
					r, g, b = 1.0, 1.0, 0.7
				end
			else
				r, g, b = UnitSelectionColor(frame.displayedUnit)
			end
		end
	end
	frame.healthBar:SetStatusBarColor(r, g, b)
	frame.selectionHighlight:SetVertexColor(r, g, b);
end)